#!/usr/bin/env python3

__copyright__ = "Copyright (C) 2021  Stefano Zacchiroli"
__license__ = "GPL-3.0-or-later"


import logging
import os
import sys

from pathlib import Path
from typing import Any

from licenseblobs import FIELD_SEP, READABLE_ENCODINGS
from licenseblobs.utils import iter_blobs, guess_mime


FIELDNAMES = [
    "sha1",
    "mime_type",
    "encoding",
    "line_count",
    "word_count",
    "size",
]


def analyze_blob(path: Path) -> list[Any]:
    size = os.stat(path).st_size
    mime_type, encoding = guess_mime(str(path))
    line_count, word_count = None, None

    if mime_type == "text/plain" and encoding in READABLE_ENCODINGS:
        line_count = 0
        word_count = 0
        try:
            with open(path, encoding="utf8") as f:
                for line in f:
                    line_count += 1
                    word_count += len(line.rstrip().split())
        except UnicodeDecodeError:
            line_count = None
            word_count = None

    return (
        FIELD_SEP.join(
            [
                path.name,
                mime_type,
                encoding,
                str(line_count) if line_count is not None else "",
                str(word_count) if word_count is not None else "",
                str(size),  # byte count
            ]
        )
        + "\n"
    )


def mine(blob_dir: Path, csv_outfile: Path) -> None:
    if os.path.exists(csv_outfile):  # ensure clean slate
        os.remove(csv_outfile)

    with open(csv_outfile, "w") as csvfile:
        csvfile.write(FIELD_SEP.join(FIELDNAMES) + "\n")
        for path in iter_blobs(blob_dir):
            csvfile.write(analyze_blob(path))


if __name__ == "__main__":
    logging.basicConfig(
        format="%(asctime)s : %(levelname)s : %(message)s", level=logging.INFO
    )
    if len(sys.argv) != 3:
        print("Usage: python -m licenseblobs.stats BLOB_DIR CSV_OUTFILE")
        sys.exit(1)

    mine(Path(sys.argv[1]), Path(sys.argv[2]))
